/* ----------------------------------------------------
 *  LISTING 1
 *
 *  Filename:           examp.c
 *  Summary:            File Manager extension DLL,
 *                      skeleton template
 *  Author:             T.W. Nelson
 *  Compiler:           BCC++ 4.0
 *  Compile options:    DLL, large model, C++ compile
 *
 * ------------------------------------------------- */

#include <wfext.h>     /* #includes windows.h */

extern "C"  {   /* descramble for C++ compile.... */
LONG WINAPI FMExtensionProc(HWND, WORD, LONG);
}

char szMenuTitle[] = "E&xample";
WORD      wDelta;          /* menu delta-factor */
HINSTANCE hFMinst;         /* FM instance handle */
HINSTANCE hEXinst;         /* DLL instance handle */
HTASK     hFMtask;         /* FM task handle */
HWND      hFMwnd;          /* FM window handle */
HMENU     hFMmenu;         /* FM's main menu */
HMENU     hExMenu;         /* our extension menu */
char msgBuf[200];

/* Menu ID numbers...... */
#define IDM_CALC        1
#define IDM_WINSIGHT    2
#define IDM_DRIVE       3
#define IDM_API         4
#define IDM_ABOUT       5

/* ------------------------------------------------- */
int CALLBACK LibMain( HINSTANCE hinst, WORD wSeg,
                      WORD wHeap, LPSTR cmdline )
{
   /* Although LibMain() isn't really needed, it
    * provides a portable method of retreiving the
    * module's instance handle w/o knowing the name of
    * the extension for call to GetModuleHandle()....
    */

    hEXinst = hinst;
    return 1;
}

int CALLBACK WEP( int exitType )
{
   /* Not really needed, but added here to let
    * MessageBeep() indicate when FM unloads the
    * DLL (maybe prematurely)........
    */

    MessageBeep(0);
    return 1;
}

HMENU SetupExtensionMenu(void)
{
   /* Create the extension menu.  This can be done
    * either dynamically, as here, or by loading a
    * menu resource with LoadMenu().......
    */

    HMENU hmenu = CreatePopupMenu();

    AppendMenu( hmenu, MF_STRING,
                       IDM_CALC, "&Calculator" );
    AppendMenu( hmenu, MF_STRING,
                       IDM_WINSIGHT, "&WinSight" );
    AppendMenu( hmenu, MF_STRING,
                       IDM_DRIVE, "&Drive Info" );
    AppendMenu( hmenu, MF_STRING,
                       IDM_API, "Windows API &Help" );
    AppendMenu( hmenu, MF_STRING,
                       IDM_ABOUT, "&About" );
    return hmenu;
}

LONG WINAPI _export
FMExtensionProc( HWND hwnd, WORD wMsg, LONG lParam )
{
    switch( wMsg )    {

    case FMEVENT_LOAD:

       /* Initialize the FMS_LOAD struct passed in
        * lParam and do other setup stuff.  FM unloads
        * the module immediately if FMS_LOAD is not
        * initialized to its satisfaction (NOTE: won't
        * call FMEVENT_UNLOAD in that case) ......
        */
        wDelta = ((LPFMS_LOAD) lParam)->wMenuDelta;
        ((LPFMS_LOAD) lParam)->dwSize =
                                 sizeof(FMS_LOAD);
        lstrcpy(((LPFMS_LOAD) lParam)->szMenuName,
                                       szMenuTitle );
        hExMenu = SetupExtensionMenu();
        ((LPFMS_LOAD) lParam)->hMenu = hExMenu;
        hFMwnd  = hwnd; /* make global copy */
        hFMinst = GetModuleHandle( "winfile.exe" );
        hFMtask = GetWindowTask( hwnd );

        return (LONG) TRUE;

    case FMEVENT_UNLOAD:
       /* Do shutdown duties, such as releasing any
        * resources allocated during setup....
        */
        DestroyMenu( hExMenu );
        break;

    case FMEVENT_SELCHANGE:
        break;

    case FMEVENT_INITMENU:
        break;

    case FMEVENT_USER_REFRESH:
        break;

    case IDM_CALC:
        WinExec( "calc", SW_SHOWNORMAL );
        break;

    case IDM_WINSIGHT:
        WinExec( "winsight", SW_SHOWNORMAL );
        break;

    case IDM_DRIVE:     {
        FMS_GETDRIVEINFO  info;

        SendMessage( hFMwnd, FM_GETDRIVEINFO, 0,
             (LONG) (LPFMS_GETDRIVEINFO) &info );
        wsprintf( msgBuf,
            "Path= %s\nFree= %ld bytes\n"
            "Total= %ld bytes\n"
            "Volume= %s\nSharePoint= %s\n",
            info.szPath, info.dwFreeSpace,
            info.dwTotalSpace, info.szVolume,
            info.szShare );
        MessageBox( hFMwnd, msgBuf,
                    "Drive Info", MB_OK );
        }

        break;

    case IDM_API:
        WinHelp( hFMwnd, "bwinapi.hlp",
                    HELP_INDEX, NULL );
        break;

    case IDM_ABOUT:
        MessageBox( hFMwnd, "File Manager Extension",
                    "About", MB_OK );
        break;

    default:
        break;
    }

    return 0L;
}

/* ----- EOF --------------------------------------- */
