// w_attr.h - definitions for useful Windows attributes
// Copyright (c) 1994 by Ron Burk

#include <commdlg.h>
#include <string.h>
#include <stdio.h>
#include "wuiman.h"
#include "wuierror.h"
#include "w_attr.h"

const char *C_TCOLORPROPERTY   = "p_Color";


AWuiObject *TWuiColorProperty::ClassConstructor(const TWuiName Name)
    {
    return new TWuiColorProperty(C_TCOLORPROPERTY);
    }

TWuiColorProperty::TWuiColorProperty(
               const TWuiName ObjectName, COLORREF *Address)
    :   TWuiLongProperty(ObjectName, (long *)Address)
    {   MEMBERASSERT();
    DefineMethod(M_EDITATTR);
    }

AWuiObject  *TWuiColorProperty::Clone(const TWuiName NewName, int Depth)
    {   MEMBERASSERT();
    AWuiObject *Result = new TWuiColorProperty(NewName);
    if(Result == NULL)
        return Result;
    else
        {
        Result->Copy(this, Depth);
        return Result;
        }
    }
void TWuiColorProperty::Copy(const AWuiObject *Other, int Depth)
    {   MEMBERASSERT();
    TWuiLongProperty::Copy(Other, Depth);
    }

TWuiName    TWuiColorProperty::ClassName() const
    {   MEMBERASSERT();
    return C_TCOLORPROPERTY;
    }

TWuiName    TWuiColorProperty::ParentClassName() const
    {   MEMBERASSERT();
    return TWuiLongProperty::ClassName();
    }

int     TWuiColorProperty::IsA(const TWuiName Class,
                                      const TWuiName Method)
    {   MEMBERASSERT();
    if(Class == C_TCOLORPROPERTY || Class == C_TLONGPROPERTY)
        {
        if(Method == "")
            return TRUE;
        else if(GetAttribute(Method) != NULL)
            return TRUE;
        else
            return FALSE;
        }
    else
        return TWuiLongProperty::IsA(Class, Method);
    }


long    TWuiColorProperty::Get(TWuiPath Path, TWuiArgs *Args,
                              char *Value, size_t MaxLength)
    {   MEMBERASSERT();
    if(AWuiObject::TraceFlag)
        Trace("TWuiColorProperty::Get", Path, Args);
    if(Args != NULL)
        {
        if(Args->AttributeName() == M_EDITATTR)
            {
            CHOOSECOLOR ColorInfo;
            COLORREF    Custom[16];
            memset(&Custom, 0, sizeof(Custom));
            memset(&ColorInfo, 0, sizeof(ColorInfo));
            ColorInfo.lStructSize   = sizeof(ColorInfo);
            ColorInfo.Flags         = CC_RGBINIT;
            ColorInfo.lpCustColors  = Custom;
            // default color is our current value
            ColorInfo.rgbResult     = PropertyValue();
            if(ChooseColor(&ColorInfo) == TRUE)
                {
                char    StringValue[64];
                sprintf(StringValue, "%ld", ColorInfo.rgbResult);
                char    MyName[TWuiName::MAXNAME+1];
                this->Name().GetName(MyName);
                return WUIMAN_Set(Path.AbsolutePath(), MyName, StringValue);
                }
            else
                return FALSE;
            }
        }
    return TWuiLongProperty::Get(Path, Args, Value, MaxLength);
    }

long    TWuiColorProperty::Set(TWuiPath Path, TWuiArgs *Args,
                                          const char *Value)
    {   MEMBERASSERT();
    if(AWuiObject::TraceFlag)
        Trace("TWuiColorProperty::Set", Path, Args);
    if(Args != NULL && Args->AttributeName() != Name())
        return TWuiLongProperty::Set(Path, Args, Value);

    char   *End;
    long    Number = strtol(Value, &End, 0);
    if(End == Value || *End != '\0')
        {
        DEBUG_ERROR("Not a legal value for a color"
                " attribute: '%s'", Value);
        return WUIMAN_ERR_SET_BAD_INTEGRAL;
        }
    if(Number & 0xFF000000L)
        {
        DEBUG_ERROR("Not a legal color value: '%s'", Value);
        return WUIMAN_ERR_SET_BAD_INTEGRAL;
        }

    return TWuiLongProperty::Set(Path, Args, Value); 
    }


